indexing
	description: "TRIO Game"
	author: "Dr. Thomas Tensi"
	date: "$Date$"
	revision: "$Revision$"

class
	GAME

create
	make

feature {NONE} -- Initialization

	make (zeroes_are_allowed : BOOLEAN;  board_width : INTEGER) is
			-- Initialize `Current'.
		do
			create reference_token_list.make(zeroes_are_allowed)
			create board.make(reference_token_list, board_width)
			search_value_count := max_search_value - min_search_value + 1
			create search_value_list.make_filled(search_value_count)
			start
		ensure
			consistent_value_list : is_consistent_value_list(search_value_list)
		end


feature -- Access
	min_search_value : INTEGER is 10
	max_search_value : INTEGER is 80

	board : BOARD  -- a trio board
	reference_token_list : BOARD_TOKEN_LIST   -- list of allowed tokens

	-- --------------------

	after : BOOLEAN is
			-- tell whether search value list is exhausted
		do
			Result := search_value_list.after
		end

	-- --------------------

	item : INTEGER is
			-- current item of search value list
		do
			Result := search_value_list.item
		end



feature -- Element change

	start is
			-- start a new game
		local
			current_time : TIME
			i : INTEGER
			j : INTEGER
			rnd : RANDOM
			temp_array : ARRAY[INTEGER]
		do
			-- set board to a random configuration
			board.reset

			-- set search value list to a permuted variant of
			-- {min_search_value,...,max_search_value}
			create temp_array.make(1, search_value_count)

			from
				i := 1
			until
				i > search_value_count
			loop
				temp_array[i] := min_search_value + i - 1
				i := i + 1
			end

			-- select randomly from temporary array and shorten its length accordingly
			create current_time.make_now
			create rnd.set_seed(current_time.seconds)
			rnd.start

			from
				i := search_value_count
			until
				i = 0
			loop
				rnd.forth
				j := (rnd.real_item * i).floor + 1
				search_value_list[i] := temp_array[j]
				temp_array[j] := temp_array[i]
				i := i - 1
			end

			search_value_list.start
		end

	-- --------------------

	forth is
			-- step to next element of search value list
		do
			search_value_list.forth
		end


feature {NONE} -- Implementation
	search_value_list : ARRAYED_LIST[INTEGER]  -- list of search values
	search_value_count : INTEGER  -- number of values to be searched for

	-- --------------------

	is_consistent_value_list (value_list : LIST[INTEGER]) : BOOLEAN is
			-- checks whether `value_list' contains all values from `min_search_value'
			-- to `max_search_value' exactly once
		local
			i : INTEGER
			is_okay : BOOLEAN
			sorted_list : SORTED_TWO_WAY_LIST[INTEGER]
		do
			create sorted_list.make
			sorted_list.fill(value_list)
			value_list.start
			sorted_list.start
			is_okay := (sorted_list.count = search_value_count)

			from
				i := min_search_value
			until
				i > max_search_value or not is_okay
			loop
				is_okay := (sorted_list.item = i)
				i := i + 1
				sorted_list.forth
			end

			Result := is_okay
		end


invariant
	invariant_clause: True -- Your invariant here

end
