indexing
	description: "Window objects for setting User-specific Game Preferences"
	author: "Dr. Thomas Tensi"
	date: "$Date$"
	revision: "$Revision$"

class
	PREFERENCES_WINDOW


inherit

	WEL_MODAL_DIALOG
		redefine
			setup_dialog,
			on_ok
		end

	RESOURCE_CONSTANTS
		export
			{NONE} all
		end


create
	make


feature {NONE} -- Initialization

	make (trio_application : ROOT_CLASS;  parent_window : WEL_WINDOW;
		  preferences : GAME_PREFERENCES) is
			-- Initialize `Current' with parent `parent_window' and referring
			-- to game settings in `preferences'
		do
			resource_manager := trio_application.resource_manager
			sound_manager    := trio_application.sound_manager

			make_by_id(parent_window, resource_manager.adapt_id(rsc_cd_id_setupdlg))

			create boardsize_combobox.make_by_id(current,
					resource_manager.adapt_id(rsc_cd_id_setupdlg_boardsize_combobox))
			create boardsize_combobox_text.make_by_id(current,
					resource_manager.adapt_id(rsc_cd_id_setupdlg_boardsize_combobox_text))
			create language_combobox.make_by_id(current,
					resource_manager.adapt_id(rsc_cd_id_setupdlg_language_combobox))
			create language_combobox_text.make_by_id(current,
					resource_manager.adapt_id(rsc_cd_id_setupdlg_language_combobox_text))
			create show_unsolvable_button_checkbox.make_by_id(current,
			        resource_manager.adapt_id(rsc_cd_id_setupdlg_unsolvable_checkbox))
			create sounds_checkbox.make_by_id(current,
			        resource_manager.adapt_id(rsc_cd_id_setupdlg_sounds_checkbox))
			create time_combobox.make_by_id(current,
					resource_manager.adapt_id(rsc_cd_id_setupdlg_time_combobox))
			create time_combobox_text.make_by_id(current,
					resource_manager.adapt_id(rsc_cd_id_setupdlg_time_combobox_text))
			create zeroes_checkbox.make_by_id(current,
			        resource_manager.adapt_id(rsc_cd_id_setupdlg_zeroes_checkbox))
			create okay_button.make_by_id(current, rsc_id_okay)
			create cancel_button.make_by_id(current, rsc_id_cancel)

			game_preferences := preferences
		end


feature -- Event Handling (public)

	setup_dialog is
		local
			enumeration_value : STRING
			language_list : LIST[STRING]
			st : STRING
		do
			-- set title
			set_text(resource_manager.get_cd_string(rsc_cd_string_setupdlg_title))

			-- fill the descriptions of the interaction elements
			st := resource_manager.get_cd_string(rsc_cd_string_setupdlg_boardsize)
			boardsize_combobox_text.set_text(st)
			st := resource_manager.get_cd_string(rsc_cd_string_setupdlg_language)
			language_combobox_text.set_text(st)
			st := resource_manager.get_cd_string(rsc_cd_string_setupdlg_sounds)
			sounds_checkbox.set_text(st)
			st := resource_manager.get_cd_string(rsc_cd_string_setupdlg_time)
			time_combobox_text.set_text(st)
			st := resource_manager.get_cd_string(rsc_cd_string_setupdlg_unsolvable)
			show_unsolvable_button_checkbox.set_text(st)
			st := resource_manager.get_cd_string(rsc_cd_string_setupdlg_zeroes)
			zeroes_checkbox.set_text(st)

			st := resource_manager.get_cd_string(rsc_cd_string_okay)
			okay_button.set_text(st)
			st := resource_manager.get_cd_string(rsc_cd_string_cancel)
			cancel_button.set_text(st)

			-- fill the combo box variants
			enumeration_value :=
			  resource_manager.get_cd_string(rsc_cd_string_setupdlg_boardsize1)
			boardsize_combobox.add_string(enumeration_value)
			enumeration_value :=
			  resource_manager.get_cd_string(rsc_cd_string_setupdlg_boardsize2)
			boardsize_combobox.add_string(enumeration_value)

			language_list := resource_manager.language_list

			from language_list.start until language_list.after loop
				language_combobox.add_string(language_list.item)
				language_list.forth
			end

			enumeration_value :=
			  resource_manager.get_cd_string(rsc_cd_string_setupdlg_time1)
			time_combobox.add_string(enumeration_value)
			enumeration_value :=
			  resource_manager.get_cd_string(rsc_cd_string_setupdlg_time2)
			time_combobox.add_string(enumeration_value)

			-- select the correct values
			select_in_combobox(boardsize_combobox, game_preferences.board_size.out)
			select_in_combobox(language_combobox, game_preferences.language.out)
			set_checkbox(sounds_checkbox, game_preferences.sound_is_on)
			select_in_combobox(time_combobox, game_preferences.think_time.out)
			set_checkbox(zeroes_checkbox, game_preferences.zeroes_are_allowed)
			set_checkbox(show_unsolvable_button_checkbox,
						 game_preferences.unsolvable_button_is_shown)
			enable_commands
		end

	-- --------------------

	on_ok is
			-- Stores settings permanently.
		local
			board_size : INTEGER
			language : STRING
			sound_is_on : BOOLEAN
			think_time : INTEGER
			unsolvable_button_is_shown : BOOLEAN
			zeroes_are_allowed : BOOLEAN
		do
			if boardsize_combobox.selected_item = 0 then
				board_size := 7
			else
				board_size := 9
			end

			if time_combobox.selected_item = 0 then
				think_time := 25
			else
				think_time := 40
			end

			zeroes_are_allowed         := zeroes_checkbox.checked
			unsolvable_button_is_shown := show_unsolvable_button_checkbox.checked
			sound_is_on                := sounds_checkbox.checked
			language		           := language_combobox.selected_string

			game_preferences.put_settings(board_size, think_time, zeroes_are_allowed,
										  sound_is_on, unsolvable_button_is_shown, language)
			resource_manager.set_language(language)
			sound_manager.set_active(sound_is_on)
			destroy
		end


feature {NONE} -- Implementation

	resource_manager : RESOURCE_MANAGER
	sound_manager : SOUND_MANAGER

	-- preferences
	game_preferences : GAME_PREFERENCES

    -- interaction elements
    boardsize_combobox : WEL_DROP_DOWN_LIST_COMBO_BOX
    boardsize_combobox_text : WEL_STATIC
    language_combobox : WEL_DROP_DOWN_LIST_COMBO_BOX
    language_combobox_text : WEL_STATIC
    show_unsolvable_button_checkbox : WEL_CHECK_BOX
    sounds_checkbox : WEL_CHECK_BOX
    time_combobox_text : WEL_STATIC
    time_combobox : WEL_DROP_DOWN_LIST_COMBO_BOX
	zeroes_checkbox : WEL_CHECK_BOX
	okay_button : WEL_PUSH_BUTTON
	cancel_button : WEL_PUSH_BUTTON

	-- --------------------

	select_in_combobox (combobox : WEL_COMBO_BOX; st : STRING) is
			-- Selects first entry with prefix `st' in `combobox'.
		local
			index : INTEGER
		do
			index := combobox.find_string(0, st)
			index := index.max (0)
			combobox.select_item(index)
		end

	-- --------------------

	set_checkbox (checkbox : WEL_CHECK_BOX; value : BOOLEAN) is
			-- Sets `checkbox' to boolean `value'.
		do
			if value then
				checkbox.set_checked
			else
				checkbox.set_unchecked
			end
		end
end
